SystemOrganization addCategory: #'Seaside-Tests-JSON'!

!GRPackage classSide methodsFor: '*seaside-tests-json' stamp: 'pmm 2/18/2012 16:34'!
seasideTestsJson
	^ self new
		name: 'Seaside-Tests-JSON';
		description: 'Unit tests for the Seaside-JSON-Core package.';
		addDependency: 'Seaside-JSON-Core';
		url: #seasideUrl;
		yourself! !

TestCase subclass: #WAJsonRenderingTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-JSON'!

!WAJsonRenderingTest methodsFor: 'private' stamp: 'pmm 1/22/2012 16:24'!
assert: aBlock gives: aString
	| json |
	json := String streamContents: [ :stream |
		| context document canvas |
		document := WAJsonDocument on: stream.
		context := WARenderContext new
			actionUrl: WAUrl new;
			document: document;
			yourself.
		 canvas := WAJsonCanvas context: context.
		canvas render: aBlock.
		document close ].
	self
		assert: json = aString
		description:	'Expected: ' , aString printString , ', Actual: ', json printString
		resumable: true! !

!WAJsonRenderingTest methodsFor: 'tests' stamp: 'pmm 2/18/2012 14:26'!
testArray
	self
		assert: [ :json |
			json array: [
				json
					value: 1;
					value: '1';
					value: true;
					value: false;
					value: nil ] ]
		gives: '[1, "1", true, false, null]'.
	self
		assert: [ :json |
			json array: [
				json value: 1 ] ]
		gives: '[1]'! !

!WAJsonRenderingTest methodsFor: 'tests' stamp: 'pmm 8/24/2011 12:03'!
testNestedArray
	self
		assert: [ :json |
			json array: [
				json
					value: 1;
					value: [
						json array: [
							json value: '1' ] ];
					value: 2 ] ]
		gives: '[1, ["1"], 2]'.
	self
		assert: [ :json |
			json array: [
				json array: [
					json value: 1 ] ] ] 
		gives: '[[1]]'! !

!WAJsonRenderingTest methodsFor: 'tests' stamp: 'pmm 8/24/2011 12:06'!
testNestedObject
	self
		assert: [ :json |
			json object: [
				json
					key: 'a' value: 'b';
					key: '1' value: [
						json object: [
							json key: '2' value: 2 ] ];
					key: 'c' value: 'd' ] ]
		gives: '{"a": "b", "1": {"2": 2}, "c": "d"}'! !

!WAJsonRenderingTest methodsFor: 'tests' stamp: 'pmm 8/24/2011 13:58'!
testObject
	self
		assert: [ :json |
			json object: [
				json
					key: '1' value: '1';
					key: '2' value: 2 ] ]
		gives: '{"1": "1", "2": 2}'.
	self
		assert: [ :json |
			json object: [
				json key: '1' value: 1 ] ]
		gives: '{"1": 1}'! !

!WAJsonRenderingTest methodsFor: 'tests' stamp: 'pmm 5/6/2012 14:51'!
testStringEscape
	self
		assert: [ :json |
			json array: [
				json value: '"\/' ] ]
		gives: '["\"\\/"]'! !

!WAJsonRenderingTest methodsFor: 'tests' stamp: 'pmm 5/6/2012 14:59'!
testStringEscapeNamed
	| backSpace formFeed lineFeed carriageReturn horizontalTab |
	backSpace := Character codePoint: 8.
	horizontalTab := Character codePoint: 9.
	lineFeed := Character codePoint: 10.
	formFeed := Character codePoint: 12.
	carriageReturn := Character codePoint: 13.
	self
		assert: [ :json |
			json array: [
				json value: (String streamContents: [ :stream |
					stream
						nextPut: backSpace;
						nextPut: horizontalTab;
						nextPut: lineFeed;
						nextPut: formFeed;
						nextPut: carriageReturn ]) ] ]
		gives: '["\b\t\n\f\r"]'! !

!WAJsonRenderingTest methodsFor: 'tests' stamp: 'pmm 5/6/2012 15:12'!
testStringEscapeUnicode
	| lineSeparator paragraphSeparator snowMan |
	lineSeparator := Character codePoint: 16r2028.
	paragraphSeparator := Character codePoint: 16r2028.
	paragraphSeparator := Character codePoint: 16r2028.
	snowMan := Character codePoint: 16r2603.
	self
		assert: [ :json |
			json array: [
				json value: (String streamContents: [ :stream |
					stream
						nextPut: lineSeparator;
						nextPut: paragraphSeparator;
						nextPut: snowMan ]) ] ]
		gives: '["', (String with: lineSeparator), (String with: paragraphSeparator), (String with: snowMan), '"]'! !

TestCase subclass: #WAJsonStreamTest
	instanceVariableNames: 'printer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Tests-JSON'!

!WAJsonStreamTest methodsFor: 'asserting' stamp: 'pmm 2/18/2012 16:06'!
assert: anObject equals: aString
	self
		assert: anObject asJson = aString
		description: anObject asJson , ' expected to be ' , aString greaseString
		resumable: true! !

!WAJsonStreamTest methodsFor: 'printing' stamp: 'pmm 2/18/2012 16:06'!
jsonOn: aStream
	printer value: aStream! !

!WAJsonStreamTest methodsFor: 'tests' stamp: 'pmm 2/18/2012 16:06'!
testArray
	self assert: #(1 2 3) equals: '[1, 2, 3]'! !

!WAJsonStreamTest methodsFor: 'tests-literals' stamp: 'pmm 2/18/2012 16:06'!
testCharacter
	self assert: $a equals: '"a"'! !

!WAJsonStreamTest methodsFor: 'tests-custom' stamp: 'pmm 2/18/2012 16:06'!
testCustomArray
	printer := [ :stream | stream render: #('aString1' ('aString2' 'aString3')) ].
	self assert: self equals: '["aString1", ["aString2", "aString3"]]'! !

!WAJsonStreamTest methodsFor: 'tests-custom' stamp: 'pmm 2/18/2012 16:06'!
testCustomDictionary
	printer := [ :stream | 
		stream render: (GRSmallDictionary new
			at: 'foo' put: 'bar';
			at: 'zork' put: (GRSmallDictionary new
				at: 'foo' put: 'bar';
				yourself);
			yourself) ].
	self assert: self equals: '{"foo": "bar", "zork": {"foo": "bar"}}'! !

!WAJsonStreamTest methodsFor: 'tests' stamp: 'pmm 2/18/2012 16:06'!
testDictionary
	self
		assert:
			((GRSmallDictionary new)
				at: 'foo' put: 1;
				at: 'bar' put: 2;
				at: 'zork' put: 3;
				yourself)
		equals: '{"foo": 1, "bar": 2, "zork": 3}'.
	self
		assert:
			((GRSmallDictionary new)
				at: '' put: 'empty';
				yourself)
		equals: '{"": "empty"}'.
	self
		assert:
			((Dictionary new)
				at: 1 put: 'not-a-string';
				yourself)
		equals: '{1: "not-a-string"}'.
	self
		assert:
			((Dictionary new)
				at: '1abc' put: 'not-an-id';
				yourself)
		equals: '{"1abc": "not-an-id"}'.
	self
		assert:
			((Dictionary new)
				at: 'abc1' put: 'totally sexy';
				yourself)
		equals: '{"abc1": "totally sexy"}'! !

!WAJsonStreamTest methodsFor: 'tests-literals' stamp: 'pmm 2/18/2012 16:06'!
testLiteral
	self assert: true equals: 'true'.
	self assert: false equals: 'false'.
	self assert: nil equals: 'null'! !

!WAJsonStreamTest methodsFor: 'tests-literals' stamp: 'pmm 2/18/2012 16:06'!
testNumber
	self assert: 123 equals: '123'.
	self assert: 123.4 equals: '123.4'.! !

!WAJsonStreamTest methodsFor: 'tests' stamp: 'pmm 2/18/2012 16:06'!
testObject
	self
		should: [ Object new asJson ]
		raise: Error! !

!WAJsonStreamTest methodsFor: 'tests-literals' stamp: 'pmm 2/18/2012 16:06'!
testStringBasic
	self assert: 'abc' equals: '"abc"'.
	self assert: '123' equals: '"123"'! !

!WAJsonStreamTest methodsFor: 'tests-literals' stamp: 'pmm 2/18/2012 16:06'!
testStringEscaping
	self assert: '''' equals: '"''"'.
	self assert: '"' equals: '"\""'.
	self assert: '\' equals: '"\\"'! !

!WAJsonStreamTest methodsFor: 'tests-literals' stamp: 'pmm 2/18/2012 16:06'!
testStringMultiByte
	"regression test for
	Issue 217: MultibyteString JavascriptOn infinite recursion [GemStone]
	http://code.google.com/p/glassdb/issues/detail?id=217"
	| multibyteString |
	multibyteString := String with: (Character codePoint: 8364).
	self assert: multibyteString equals: '"', multibyteString, '"'.
! !

!WAJsonStreamTest methodsFor: 'tests-literals' stamp: 'pmm 2/18/2012 16:06'!
testStringSpecial
	self assert: (String with: Character cr) equals: '"\r"'.
	self assert: (String with: Character lf) equals: '"\n"'.
	self assert: (String with: Character tab) equals: '"\t"'.

	self assert: (String with: (Character codePoint: 0)) equals: '"\0"'.
	self assert: (String with: (Character codePoint: 1)) equals: '"\x01"'.
	self assert: (String with: (Character codePoint: 31)) equals: '"\x1f"'! !
